package de.scrum_master.aspect;

import java.lang.reflect.Method;

import org.apache.commons.lang3.ClassUtils;
import org.aspectj.lang.Signature;
import org.aspectj.lang.SoftException;

public aspect MethodTracer pertarget(call(* *(..))) {
	public MethodTracer() {
		super();
		System.out.printf(
				"%n%-60s  |  %-30s  |  %-30s  |  %-30s%n",
				"Join point",
				"AJ target class",
				"AJ method declaring type",
				"Real method declaring type"
			);
		System.out.printf(
				"%-60s==|==%-30s==|==%-30s==|==%-30s%n",
				"============================================================",
				"==============================",
				"==============================",
				"=============================="
			);
	}

	before(Object target) : !within(MethodTracer) && call(* *(..)) && target(target) {
		Signature methodSignature = thisJoinPointStaticPart.getSignature();
		Method method;
		try {
			String[] signatureParts = methodSignature.toLongString().split("[()]|, ");
			Class<?>[] parameterTypes = new Class<?>[signatureParts.length - 1];
			int index = -1;
			for (String signaturePart : signatureParts) {
				if (++index == 0)
					continue;
				parameterTypes[index - 1] = ClassUtils.getClass(signaturePart);
			}
			method = target.getClass().getMethod(methodSignature.getName(), parameterTypes);
		} catch (ClassNotFoundException|NoSuchMethodException e) {
			throw new SoftException(e);
		}
		System.out.printf(
			"%-60s  |  %-30s  |  %-30s  |  %-30s%n",
			thisJoinPointStaticPart,
			target.getClass().getName(),
			methodSignature.getDeclaringTypeName(),
			method.getDeclaringClass().getName()
		);
	}
}
